<?php

/**
 * @file
 * Admin setting for Shipwire Ubercart module.
 */

/**
 * Admin form for shipwire ubercart settings. 
 */
function shipwire_ubercart_admin_settings() {
  $form = array();

  if (!Shipwire::operational()) {
    $form['configure'] = array(
      '#markup' => t('Shipwire account settings must be configured first.') . ' '
      . l(t('Configure Shipwire'), 'admin/shipwire/settings'),
    );
    return system_settings_form($form);
  }
  $form['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('General'),
    '#collapsible' => TRUE,
  );
  $form['general']['shipwire_ubercart_order_prefix'] = array(
    '#type' => 'textfield',
    '#title' => t('Order ID prefix'),
    '#description' => t('To ensure unique order numbers please specify an'
      . ' order prefix. This will be added to the beginning of the Ubercart'
      . ' order number e.g. Prefix "SU-" and Ubercart order number "11573" form'
      . ' Order id "SU-11573". The prefix must include a non numeric character.'),
    '#default_value' => variable_get('shipwire_ubercart_order_prefix', 'SU-'),
    '#size' => 5,
    '#required' => TRUE,
  );
  $form['general']['shipwire_ubercart_log_order'] = array(
    '#type' => 'radios',
    '#title' => t('Save shipment status messages to order log'),
    '#default_value' => variable_get('shipwire_ubercart_log_order', 1),
    '#options' => array(1 => t('Yes'), 0 => t('No')),
  );

  if (user_access('configure quotes')) {
    $currency_code = variable_get('uc_currency_code', 'USD');
    $currency_sign = variable_get('uc_currency_sign', '$');
    $currency_sign_after = variable_get('uc_sign_after_amount', FALSE);

    $field_prefix = '';
    $field_suffix = '';
    if ($currency_sign_after) {
      $field_suffix = $currency_sign . ' (' . $currency_code . ')';
    }
    else {
      $field_prefix = $currency_sign;
      $field_suffix = $currency_code;
    }

    $form['rate'] = array(
      '#type' => 'fieldset',
      '#title' => t('Shipping rate quote'),
      '#collapsible' => TRUE,
    );
    $form['rate']['shipwire_ubercart_padding_rate'] = array(
      '#type' => 'textfield',
      '#title' => t('Delivery rate padding'),
      '#description' => t('Additional handling fee towards added to the quote'
        . ' returned from Shipwire.'),
      '#default_value' => variable_get('shipwire_ubercart_padding_rate', 0),
      '#field_prefix' => $field_prefix,
      '#field_suffix' => $field_suffix,
      '#size' => 10,
    );
    $form['rate']['shipwire_ubercart_padding_days'] = array(
      '#type' => 'textfield',
      '#title' => t('Delivery days padding'),
      '#description' => t('Additional days added to the estimated shipping'
        . ' delivery date.'),
      '#default_value' => variable_get('shipwire_ubercart_padding_days', 0),
      '#field_suffix' => t('Days'),
      '#size' => 10,
    );
    $form['rate']['shipwire_ubercart_exclude'] = array(
      '#type' => 'textarea',
      '#title' => t('Exclude rates when order contains product SKUs'),
      '#description' => t('Enter a comma separated list of product SKUs if you'
        . ' wish Shipwire to exclude shipping rate quotes when the order contains'
        . ' the products. Only use this feature if you have another shipping rate'
        . ' module installed or customers will not be able to complete checkout.'),
      '#default_value' => variable_get('shipwire_ubercart_exclude', ''),
    );
  }

  $form['fulfill'] = array(
    '#type' => 'fieldset',
    '#title' => t('Order fulfillment'),
    '#collapsible' => TRUE,
  );
  $form['fulfill']['shipwire_ubercart_fulfill_type'] = array(
    '#type' => 'radios',
    '#title' => t('Order fulfillment method'),
    '#description' => t('Choose whether fulfillment should take place'
      . ' immediately using rules or using batch processing on CRON runs.'
      . ' Immediate execution requires communication with Shipwire during the'
      . ' checkout process which will slow down checkout and possibly fail'
      . ' causing checkout to stop. Batch processing on CRON runs reduces the'
      . ' number of requests to Shipwire, as many orders are submitted in one'
      . ' request, and ensures successful checkout even if communication is'
      . ' temporarily interrupted. Batch processing is recommended on sites that'
      . ' have more than 60 orders per hour.'),
    '#options' => array(
      'now' => t('Immediately during checkout using Rules'),
      'later' => t('Batch submission on CRON run'),
    ),
    '#default_value' => variable_get('shipwire_ubercart_fulfill_type', 'now'),
  );

  $stock_available = module_exists('uc_stock');
  $inventory_description = '';
  if (!$stock_available) {
    $inventory_description = t('Inventory stock updates require the Ubercart'
      . ' Stock module (uc_stock). Please enable the module if you want to use'
      . ' this feature.');
  }
  $form['inventory'] = array(
    '#type' => 'fieldset',
    '#title' => t('Inventory'),
    '#description' => $inventory_description,
    '#collapsible' => TRUE,
  );
  $form['inventory']['shipwire_ubercart_inventory_update'] = array(
    '#type' => 'checkbox',
    '#title' => t('Update product stock levels'),
    '#description' => t('When enabled product stock quantity will be checked'
      . ' in Shipwire warehouses and will update this site. If this store'
      . ' maintains stock outside of Shipwire then do not enable this feature.'),
    '#default_value' => variable_get('shipwire_ubercart_inventory_update', 0),
    '#disabled' => !$stock_available,
  );

  $form['#validate'][] = 'shipwire_ubercart_admin_settings_validate';
  return system_settings_form($form);
}

/**
 * Validation callback for shipwire ubercart settings form.
 */
function shipwire_ubercart_admin_settings_validate($form, &$form_state) {
  $fsv = $form_state['values'];

  $prefix = preg_replace('/[0-9]/', '', $fsv['shipwire_ubercart_order_prefix']);
  if (empty($prefix)) {
    form_set_error('shipwire_ubercart_order_prefix', t('Order prefix must contain non numeric chracters.'));
  }

  if (preg_match('/[^0-9]/', $fsv['shipwire_ubercart_padding_days'])) {
    form_set_error('shipwire_ubercart_padding_days', t('Delivery days padding must be a whole number.'));
  }

  $dec = variable_get('uc_currency_dec', '.');
  if ($dec == '.') {
    $dec = '\\' . $dec;
  }
  $padding_rate = preg_replace("/[^0-9$dec]/", '', $fsv['shipwire_ubercart_padding_rate']);
  if (!is_numeric($padding_rate)) {
    form_set_error('shipwire_ubercart_padding_rate', t('Delivery rate padding must be a number.'));
  }
  elseif ((float) $padding_rate < 0) {
    form_set_error('shipwire_ubercart_padding_rate', t('Delivery rate padding must be 0 or greater.'));
  }
}
